"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DOMRenderer = void 0;
var tslib_1 = require("tslib");
var web_utility_1 = require("web-utility");
var VDOM_1 = require("./VDOM");
var DOMRenderer = /** @class */ (function () {
    function DOMRenderer() {
        var _this = this;
        this.eventPattern = /^on\w+/;
        this.keyOf = function (_a, index) {
            var key = _a.key, text = _a.text, props = _a.props, selector = _a.selector;
            return key || (props === null || props === void 0 ? void 0 : props.id) || text || (selector && selector + index);
        };
        this.vNodeOf = function (list, key) {
            return list.find(function (vNode, index) { return _this.keyOf(vNode, index) + '' === key; });
        };
    }
    DOMRenderer.prototype.updateProps = function (node, oldProps, newProps, onDelete, onAdd) {
        if (oldProps === void 0) { oldProps = {}; }
        if (newProps === void 0) { newProps = {}; }
        var group = (0, web_utility_1.diffKeys)(Object.keys(oldProps), Object.keys(newProps)).group;
        for (var _i = 0, _a = group[web_utility_1.DiffStatus.Old] || []; _i < _a.length; _i++) {
            var key = _a[_i][0];
            onDelete === null || onDelete === void 0 ? void 0 : onDelete(node, key);
        }
        for (var _b = 0, _c = tslib_1.__spreadArray(tslib_1.__spreadArray([], (group[web_utility_1.DiffStatus.Same] || []), true), (group[web_utility_1.DiffStatus.New] || []), true); _b < _c.length; _b++) {
            var key = _c[_b][0];
            if (oldProps[key] !== newProps[key])
                if (onAdd instanceof Function)
                    onAdd(node, key, newProps[key]);
                else
                    Reflect.set(node, key, newProps[key]);
        }
    };
    DOMRenderer.prototype.createNode = function (vNode) {
        if (vNode.text)
            return (vNode.node = document.createTextNode(vNode.text));
        vNode.node = vNode.tagName
            ? document.createElement(vNode.tagName)
            : document.createDocumentFragment();
        return this.patch({ tagName: vNode.tagName, node: vNode.node }, vNode)
            .node;
    };
    DOMRenderer.prototype.updateChildren = function (node, oldList, newList) {
        var _this = this;
        var _a, _b;
        var _c = (0, web_utility_1.diffKeys)(oldList.map(this.keyOf), newList.map(this.keyOf)), map = _c.map, group = _c.group;
        for (var _i = 0, _d = group[web_utility_1.DiffStatus.Old] || []; _i < _d.length; _i++) {
            var key = _d[_i][0];
            (_b = (_a = this.vNodeOf(oldList, key)) === null || _a === void 0 ? void 0 : _a.node) === null || _b === void 0 ? void 0 : _b.remove();
        }
        var newNodes = newList.map(function (vNode, index) {
            var key = _this.keyOf(vNode, index);
            if (map[key] === web_utility_1.DiffStatus.Same)
                return _this.patch(_this.vNodeOf(oldList, key), vNode).node;
            return _this.createNode(vNode);
        });
        node.append.apply(node, newNodes);
    };
    DOMRenderer.prototype.patch = function (oldVNode, newVNode) {
        var _this = this;
        this.updateProps(oldVNode.node, oldVNode.props, newVNode.props, function (node, key) {
            return _this.eventPattern.test(key)
                ? (node[key.toLowerCase()] = null)
                : node.removeAttribute(VDOM_1.VDOMNode.propsMap[key] || key);
        }, function (node, key, value) {
            return (node[_this.eventPattern.test(key) ? key.toLowerCase() : key] =
                value);
        });
        this.updateProps(oldVNode.node.style, oldVNode.style, newVNode.style, function (node, key) { return node.removeProperty(key); });
        this.updateChildren(oldVNode.node, oldVNode.children || [], newVNode.children || []);
        newVNode.node = oldVNode.node;
        return newVNode;
    };
    DOMRenderer.prototype.render = function (vNode, node) {
        if (node === void 0) { node = document.body; }
        var root = new VDOM_1.VDOMNode(node);
        return this.patch(root, tslib_1.__assign(tslib_1.__assign({}, root), { children: [vNode] }));
    };
    return DOMRenderer;
}());
exports.DOMRenderer = DOMRenderer;
